# AntarcticaVorticityBudget

This repository contains codes and processed datasets for a manuscript entitled **"The large-scale vorticity balance of the Antarctic continental margin in a fine-resolution global simulation"**, by A. Palóczy, J. L. McClean, S. T. Gille and He Wang, [published on the Journal of Physical Oceanography](https://journals.ametsoc.org/jpo/article/doi/10.1175/JPO-D-19-0307.1/348316/The-large-scale-vorticity-balance-of-the-Antarctic). This [Jupyter notebook](https://nbviewer.jupyter.org/github/apaloczy/AntarcticaVorticityBudget/blob/master/index.ipynb) provides an overview of the contents.

The directory **plot_figs/** contains the Python codes used to produce the tables and figures in the manuscript (Figures 1-9) and the svg figure for the schematic (Figure 10). The codes depend on the data files in the **data_reproduce_figs/** directory. Some of these are too large to be included in this repository, but are available for download from the links listed on the accompanying README files. Please contact [André Palóczy](mailto:apaloczy@ucsd.edu) if you have issues downloading the files.

## Abstract
The depth-integrated vorticity budget of a global, eddy-permitting ocean/sea-ice simulation over the Antarctic Continental Margin (ACM) is diagnosed to understand the physical mehanisms implicated in meridional transport. The leading-order balance is between the torques due to lateral friction, nonlinear effects, and bottom vortex stretching, although details vary regionally. Maps of the time-averaged depth-integrated vorticity budget terms and time series of the spatially-averaged, depth-integrated vorticity budget terms reveal that the flow in the Amundsen, Bellingshausen and Weddell Seas and, to a lesser extent, in the western portion of East Antarctica, is closer to an approximate Topographic Sverdrup Balance (TSB) compared to other segments of the ACM. Correlation and coherence analyses further support these findings, and also show that inclusion of the vorticity tendency term in the response (the planetary vorticity advection and the bottom vortex stretching term) increases the correlation with the forcing (the vertical net stress curl), and also increases the coherence between forcing and response at high frequencies across the ACM, except for the West Antarctic Peninsula. These findings suggest that the surface-stress curl, imparted by the wind and the sea ice, has the potential to contribute to the meridional, approximately cross-slope, transport to a greater extent in the Amundsen, Bellingshausen, Weddell and part of the East Antarctic continental margin than elsewhere in the ACM.

## Authors
* [André Palóczy](http://scrippsscholars.ucsd.edu/apaloczy) (<apaloczy@ucsd.edu>)
* [Julie L. McClean](http://scrippsscholars.ucsd.edu/jmcclean)
* [Sarah T. Gille](http://scrippsscholars.ucsd.edu/sgille)
* He Wang

## Acknowledgments
A.P., J.L.M and S.T.G. gratefully acknowledge support from the US Department of Energy (DOE, grants DE-SC0014440 and DE-SC0020073) and high-performance computing support from Yellowstone (ark:/85065/d7wd3xhc) provided by NCAR's Climate Simulation Laboratory (CSL), sponsored by the National Science Foundation (NSF) and other agencies. J.L.M. was supported by an earlier U.S. DOE Office of Science grant entitled ``Ultra-High Resolution Global Climate Simulation" via a Los Alamos National Laboratory subcontract to carry out the POP/CICE simulation; both Caroline Papadopoulos (SIO/UCSD) and Elena Yulaeva (UCSD) participated in its production. S.T.G. also acknowledges NSF awards PLR-1425989 and OCE 1658001. The analyses of the model output performed in this study were enabled by computing resources provided by Oak Ridge Leadership Computing Facility (OLCF). We are also grateful to Andrew Stewart (University of California Los Angeles) and Matt Mazloff (SIO/UCSD), for insightful exchanges and Steve Yeager (National Center for Atmospheric Research, NCAR) for the additional POP code needed to archive non-standard terms for the budget as well as an NCAR Command Language (NCL) vorticity budget code that provided a starting place for our expanded Fortran-based budget code used in this study. Thoughtful critiques and comments from two anonymous reviewers substantially improved the quality of the manuscript. Reduced datasets and code necessary to reproduce the results are available at [https://github.com/apaloczy/AntarcticaVorticityBudget](https://github.com/apaloczy/AntarcticaVorticityBudget).
